@@ -19,7 +19,7 @@
 
 
 #ifndef MAX_TAG_SIZE
-#define MAX_TAG_SIZE 65536
+#define MAX_TAG_SIZE 256
 #endif
 
 
@@ -46,4 +46,15 @@ int isScriptClosingTag (char *tag);
 
 
 
+/*
+ * Checks if a given tag is an actual HTML tag
+ * It checks in a case-insensitive manner.
+ *
+ *   Given:   a string that could be an HTML tag.
+ *   Return:  1 if that tag is an HTML tag, 0 otherwise
+ */
+int isRealHtmlTag (char *tag);
+
+
+
 #endif
