@@ -15,7 +15,9 @@
  * reads from stdin.  It writes all output to stdout.
  */
 
-
+#include <ctype.h>
+#include <malloc.h>
+#include <string.h>
 
 #include "ops.h"
 #include "esc.h"
@@ -32,9 +34,11 @@ int main(int argc, char *argv[]) {
    * Variables local to the program.
    */
   FILE *inStream;
-  char tag[MAX_TAG_SIZE];
+  char *tag;
+  char *tmp;
+  int tag_size;
   char ch;
-  int i;
+  int i, j;
 
 
 
@@ -55,6 +59,16 @@ int main(int argc, char *argv[]) {
       return 0;
     }
 
+  /*
+   * Allocate tag space, now that we know we need to do some actual work.
+   */
+  tag_size = MAX_TAG_SIZE;
+  tag = (char *)malloc(tag_size);
+  if (!tag) {
+     fprintf (stderr, "Cannot malloc tag space (%d bytes).\n", tag_size);
+     return 1;
+  }
+
   /*
    * If an input file was specified, try to open a read stream on it.
    */
@@ -95,17 +109,52 @@ int main(int argc, char *argv[]) {
      * If we get this far, we've hit an HTML tag.  Read it into the
      * variable tag.
      */
+    memset(tag, 0, tag_size);
     i = 0;
     while (ch != EOF) {
       tag[i] = ch;
+      if (i == 1 && ch != '/' && !isalpha(ch)) {
+	 m_putchar(tag[0]);
+	 m_putchar(ch);
+         break;
+      }
       i++;
-      if (ch == '>')
-        break;
+      if (ch == '>') {
+	/*
+	 * If it's really an html tag, then toss it.  Otherwise, it could
+	 * have been just a '<' sign in the text.
+	 */
+	if (!isRealHtmlTag(tag)) {
+	  fprintf(stderr, "not: %s\n",tag);
+	  for (j = 0; j < i; j++)
+            m_putchar(tag[j]);
+	}
 
-      if (i > (MAX_TAG_SIZE - 1)) {
-        fprintf (stderr, "Error: encountered a tag larger than %d.\n", MAX_TAG_SIZE);
-        fprintf (stderr, "Recompile with a larger MAX_TAG_SIZE value.\n");
-        return 1;
+	if (tag_size > MAX_TAG_SIZE) {
+	  free(tag);
+	  tag_size = MAX_TAG_SIZE;
+	  tag = (char *)malloc(tag_size);
+	  if (!tag) {
+            fprintf (stderr, "Cannot malloc tag space (%d bytes).\n", tag_size);
+            return 1;
+	  }
+	}
+
+        break;
+      }
+
+      if (i > (tag_size - 1)) {
+	while (tag_size < i)
+	  tag_size *= 2;
+	tmp = (char *)malloc(tag_size);
+        if (!tmp) {
+          fprintf (stderr, "Cannot malloc tag space (%d bytes).\n", tag_size);
+          return 1;
+        }
+	memset(tmp, 0, tag_size);
+	memcpy(tmp, tag, i);
+	free(tag);
+        tag = tmp;
       }
 
       ch = fgetc (inStream);
@@ -169,16 +218,25 @@ int main(int argc, char *argv[]) {
          * Hit a '<'.  Read till we hit a '>' or a '<', filling the buffer.
          */
         i = 1;
+	memset(tag, 0, tag_size);
         tag[0] = '<';
         ch = fgetc (inStream);
         while (ch != EOF) {
           tag[i] = ch;
           i++;
 
-          if (i > MAX_TAG_SIZE) {
-            fprintf (stderr, "Error: encountered a tag larger than %d.\n", MAX_TAG_SIZE);
-            fprintf (stderr, "Recompile with a larger MAX_TAG_SIZE value.\n");
-            return 1;
+          if (i > (tag_size - 1)) {
+	    while (tag_size < i)
+	      tag_size *= 2;
+	    tmp = (char *)malloc(tag_size);
+            if (!tmp) {
+              fprintf (stderr, "Cannot malloc tag space (%d bytes).\n", tag_size);
+              return 1;
+            }
+	    memset(tmp, 0, tag_size);
+	    memcpy(tmp, tag, i);
+	    free(tag);
+            tag = tmp;
           }
 
           if ((ch == '>') || (ch == '<'))
